@@ -23,10 +23,10 @@ import logging
 
 
 class Regression():
-    def __init__(self, X:pd.DataFrame, Y, model=None, sample_weight = None, max_related_level=1, iter_num=20,
+    def __init__(self, X:pd.DataFrame, Y, init_in_vars=[], model=None, sample_weight = None, max_related_level=1, iter_num=20,
                  is_train_test_split=True, test_size=0.4, split_num=10, kw_measure_args={},
                  measure = 'ks', max_pvalue_limit=0.05, max_vif_limit=10, max_corr_limit=0.6,seed=666,
-                n_core=None, logger_info=True, logger_file=None):
+                n_core=None, logger_info=True, logger_file=None, observation_model='normal'):
         # fit_weight = None, measure_weight = None, kw_measure_args = None,
         # kw_algorithm_class_args = None
 
@@ -62,6 +62,8 @@ class Regression():
         self.logger_info = logger_info
         self.logger_file=logger_file
         self.seed=seed
+        self.in_vars=init_in_vars # 初始默认输入特征
+        self.observation_model=observation_model # 'all'代表全变量测试，观测所有特征的效果趋势
 
         if n_core is None:
             self.n_core = os.cpu_count() - 1
@@ -180,7 +182,7 @@ class Regression():
             out_vars : dict,剔除的变量列表及删除原因，其结构为:{'变量名称':[删除原因]}
         '''
         iter_idx = 0
-        in_vars = []
+        in_vars = self.in_vars
         current_perf = -np.inf
         logger_ch, fh_ch = _Tool.make_logger(self.logger_info,self.logger_file)
         while (True):
@@ -215,6 +217,8 @@ class Regression():
             if len(in_vars) == 0:
                 if logger_ch:
                     logger_ch.info('没有变量能够进入模型，建模结束')
+                if self.observation_model=='all':
+                    self.fit_observation(iter_idx, in_vars, logger_ch)
                 break
             with Pool(self.n_core) as pool:
                 result = pool.map_async(self._rm_var, zip(in_vars, repeat((in_vars, current_perf))))
@@ -226,6 +230,8 @@ class Regression():
             if (add_var is None) and (rm_var is None):
                 if logger_ch:
                     logger_ch.info('在此轮迭代中，在满足使用者所设置条件的前提下，已经不能通过增加或删除变量来进一步提升模型的指标，建模结束')
+                if self.observation_model=='all':
+                    self.fit_observation(iter_idx, in_vars, logger_ch)
                 break
             if logger_ch:
                 logger_ch.info('此轮迭代完成，当前入模变量为：%s。 当前模型性能%s为:%f' % (in_vars, self.measure, current_perf))
@@ -239,6 +245,48 @@ class Regression():
             logger_ch.removeHandler(fh_ch)
         return in_vars, del_vars
 
+
+    def fit_observation(self,iter_idx,_in_vars,logger_ch):
+        in_vars = _in_vars.copy()
+        if logger_ch:
+            logger_ch.info('****************迭代轮数(全量观察)：%s********************' % in_vars)
+        while (True):
+            iter_idx += 1
+            if iter_idx > self.iter_num:
+                break
+            out_vars = list(self.columns[~self.columns.isin(in_vars)])
+            if len(out_vars) == 0:
+                if logger_ch:
+                    logger_ch.info('(全量观察)变量全部进入模型，建模结束')
+                break
+            with Pool(self.n_core) as pool:
+                result = pool.map_async(self._add_observation_var, zip(out_vars, repeat((in_vars, -1))))
+                add_rm_var_arr = result.get()
+            add_var, rm_var_0, perf = sorted(add_rm_var_arr, key=lambda x: x[2])[-1]
+            if add_var is not None:
+                if isinstance(add_var,list):
+                    in_vars.extend(add_var)
+                else:
+                    in_vars.append(add_var)
+                current_perf = perf
+            if logger_ch:
+                logger_ch.info('此轮迭代完成(全量观察)%d，入模变量为：%s。 当前模型性能%s为:%f' % (iter_idx, add_var, self.measure, current_perf))
+
+    def _add_observation_var(self, args):
+        col = args[0]
+        in_vars, current_perf = args[1]
+        add_rm_var = (None, None, current_perf)
+        if isinstance(col,list):
+            tmp_cols = []
+            tmp_cols.extend(col)
+        else:
+            tmp_cols = [col]
+        tmp_cols.extend(in_vars)
+
+        check_pvalue, check_perf, perf, check_vif, vif, check_corr, corr_max = self._check(tmp_cols,current_perf)
+        add_rm_var = (col, None, perf)
+        return add_rm_var
+
     def _check_corr(self,col):
         for i in col:
             assert i in self.columns,'{} 不是有效字段'.format(i)
